c=======================================================================
c***********************************************************************
      subroutine set_frac(ngt)
c***********************************************************************
c=======================================================================

c=======================================================================
c     This routine reads the different environmental inputs as well
c     as vegetation characteristics and sets them in the right form.
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none

      include './com_18/parameter.common'
      include './com_18/annee.common'
      include './com_18/climin0.common'
      include './com_18/climkop.common'
      include './com_18/coord.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/eco.common'
      include './com_18/ecoin.common'
      include './com_18/ecopro.common'
      include './com_18/envi.common'
      include './com_18/fgr.common'
      include './com_18/frac_change.common'      
      include './com_18/gridclim.common'
      include './com_18/griddata.common'
      include './com_18/gridin2.common'
      include './com_18/input_par.common'
      include './com_18/inidata.common'
      include './com_18/landuse.common'      
      include './com_18/nspc.common'
      include './com_18/prev_yr.common'      
      include './com_18/radcst.common'
      include './com_18/smrd.common'
      include './com_18/sr_par.common'
      include './com_18/temper.common'
      include './com_18/vegfr.common'
      include './com_18/xlaic.common'
 
      real*4 plant_lai, plant_fgrn, frc_tree
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer , intent(in) :: ngt
      
      integer ip,k
      real*4 err,permdpt,ta_yr

c------------------------------------------------  

c=======================================================================
c This subroutine sets vegetation fraction, LAI, albedo and emissivity
c parameters, rooting depth and fsi,fci,wpi for IBM
c=======================================================================

c=======================================================================
c   sets vegetation fraction cover and LAI
c   frac(ip): vegetation fraction in each storey
c   frc(ip): fractional cover of vegetation class ip
c   fveg: fractional vegetation cover 
c=======================================================================
      do ip = 1, npft
        frac(ip) = yfrac_ini(ip,ngt)
        xlmin(ip) = ylaimin_ini(ip,ngt)
        xlmax(ip) = ylaimax_ini(ip,ngt)
        plant_lai = ylaimax_ini(ip,ngt)
        call fgrn_calc(plant_lai,plant_fgrn) 
        fgrnmax(ip) = plant_fgrn 
	xlaimax_prv(ip) = ylaimax_ini(ip,ngt)       
      end do

      do ip = npft0+1, npft
        frc(ip) = fgrnmax(ip)*frac(ip)
      end do

      frc_tree = 0.
      do ip = nherb+nbush+1, npft0
        frc(ip) = fgrnmax(ip)*frac(ip)
        frc_tree = frc_tree+frc(ip)
      end do
      if (frc_tree.gt.frac_nat(ngt)) frc_tree = frac_nat(ngt)

      do ip = 1, nherb+nbush 
        frc(ip) = (frac_nat(ngt)-frc_tree)*fgrnmax(ip)*frac(ip)
      end do

       fveg=0.
       do ip =1,npft
          fveg = fveg+frc(ip)
       enddo

       if(fveg.gt.1.) then
          err = fveg-1.
           if(err.gt.0.03)then
            write(61,*) 'Error fveg:', fveg,ylongi,ylati
            stop
           endif          
          do ip=1,npft
             frc(ip) = frc(ip)/fveg
          enddo
          fveg=1.
       elseif(fveg.lt.0.) then
          write(61,*) 'Error fveg:', fveg,ylongi,ylati
          stop
       endif

c   albv: albedo of vegetation cover
c   emiv: emissivity of vegetation cover
c  ---------------------------------------------------------------------
       albv=0.
       emiv=0.
       if(fveg.gt.1.d-7)then
         do ip=1,npft
           albv = albv + 0.5 * (alvsw(ip)+alvlw(ip))
     1                * frc(ip)/fveg
           emiv = emiv + emv(ip)*frc(ip)/fveg
         enddo
       else
         albv=0.20
         emiv=0.97
       endif

c=======================================================================
c   sdens : Density of soil mineral particles (g m-3) (not used)
c   rootd : Rooting depth (mm)
c   fsi   : Water saturation (mm) 
c   fci   : Field capacity (mm) 
c   wpi   : Wilting point (mm)
c=======================================================================

c      sdens=(cla*2.52+sil*2.60+san*2.8) * 1.d+6

      if(isunit.eq.4)then
c  lithosol: by convention rootd = 100 mm 
c  ---------------------------------------------------------------------
        rootd=500.
      else
c  rooting depth: calculated on vegetated fraction only
c  ---------------------------------------------------------------------
         if(fveg.gt.1.d-7)then
            rootd = 0.
            do ip = 1, npft
               rootd = rootd+frc(ip)*rdveg(ip)/fveg
            enddo
         else
            rootd = 500.
         endif
      endif
         
c  correction of rooting depth for permafrost
c  rooting depth set to 580 mm in permafrost regions
c  ---------------------------------------------------------------------
          
      ta_yr = 0.
      permdpt = 580.
      do k =1,nm
         ta_yr=ta_yr+tcelkop(k)/float(nm)
      enddo

      drn_fac = 1.
      if (ta_yr.lt.0.) then
         if (rootd.gt.permdpt) rootd=permdpt
         drn_fac = 0.1
      endif

c=======================================================================
c     saturation, field capacity and wilting point (mm)
c=======================================================================

      fsi=smfs*rootd
      fci=smfc*rootd
      wpi=smwp*rootd

      return
      end

c=======================================================================
c***********************************************************************
      subroutine fgrn_calc(plant_lai,plant_fgrn)
c***********************************************************************
c======================================================================= 
      implicit none
      include './com_18/parameter.common'
      include './com_18/radcst.common'
      real*4 , intent(in) :: plant_lai
      real*4 , intent(out) :: plant_fgrn
C      real*4 plant_lai,plant_fgrn     

      plant_fgrn = 1-exp(-0.5*clump_fac*plant_lai)
      if (plant_fgrn.lt.0.) plant_fgrn = 0.
             
      return
      end

c###########################################################################
      
c=======================================================================
c***********************************************************************
      subroutine drainage(stept)
c***********************************************************************
c=======================================================================

c      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/drain.common'
      include './com_18/eco.common'
      include './com_18/smrd.common'
      real*4, intent(in) :: stept
      
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer j,kf,ks0,ni,num
      real*8  aa,aj,bb,bj,cj,dfn,drai,drdf,drdfb,drdft,drk0,dsm,ermax
     &      ,err,facrun,fnet,fnt,rdep,sl,sm,sm0,sma,smb,smfsi,smpo,smwpi
     &      ,taua,taub,taucri,taum,taumax,xi,yi
c
c-----------------------------------------------      
      
      dimension xi(nfmean),yi(nfmean),fnt(0:nfn)
     1         ,aj(0:nde),bj(0:nde),cj(nde+1),sl(nde+1,nde+2)
     2         ,drai(0:nfn),drdf(nfn-1)

      drk0 = 240.

      dfn = 1.
      fnt(nfmean) = 0.
      do kf = 1,nfmean
         if(kf.le.5)then
            fnt(nfmean+kf) = dfn*(2.**kf)
         else
            fnt(nfmean+kf) = fnt(nfmean+kf-1) * 1.5
         endif
         fnt(nfmean-kf) = -fnt(nfmean+kf)
      enddo
      fntmax = fnt(nfn)

      ni = 10
      ermax = 1.d-3*stept
      taumax = 1.5*stept
      taucri = (taumax+stept)/2.

         rdep = rootd
         aa = acd
         bb = bcd
         smwpi = smwp
         smfsi = smfs
         dsm = 1./float(nwa)

         do ks0 = 0,nwa

            wa0(ks0) = float(ks0)*dsm
            sm0 = smwpi+(smfsi-smwpi)*wa0(ks0)

            do kf = 0,nfn

               fnet = fnt(kf)
               if(fnet.le.0.)then
                  smpo = 0.
               else
                  smpo = bb/(dlog(fnet/drk0)-aa)
               endif

               sma = sm0
               if (smpo.lt.smwpi) then
                  smb = smwpi
                  facrun = 0.
               elseif (smpo.gt.smfsi) then
                  smb = smfsi
                  facrun = 1.
               else
                  smb = smpo
                  facrun = 1.
               endif
               taua = 0.
               call TAU(aa,bb,fnet,rdep,drk0,sm0,smb,taub,ni)
               if(taub.gt.taumax)taub=taumax
               if (taub.le.stept) then
                  sm = smb
                  drai(kf) = fnet*taub - rdep*(sm-sm0)
     &                     + facrun*(stept-taub)*drk0*dexp(aa+bb/sm)
                  goto 2000
               endif
               taum = 0.
               do num = 1,100
                  if(taum.le.taucri)then 
                     sm = sma + (smb-sma)
     &                  * dsqrt((stept-taua)/(taub-taua))
                  else
                     sm = sma + (smb-sma)
     &                  * (((stept-taua)/(taub-taua))**2.)
                  endif  
                  call TAU(aa,bb,fnet,rdep,drk0,sm0,sm,taum,ni)
                  if (taum.gt.taumax) taum=taumax
                  err = dabs(taum-stept)
                  if (err.le.ermax) then
                     drai(kf) = fnet*taum - rdep*(sm-sm0)
                     goto 2000
                  endif
                  if (taum.le.stept)then
                     sma = sm
                     taua = taum
                  else
                     smb = sm
                     taub = taum
                  endif
               enddo

               write(28,*)'SUBROUTINE drainage: too much iterations'
               write(28,*)'num:',num
               write(28,*)'sma:',sma,' taua:',taua
               write(28,*)'smb:',smb,' taub:',taub
               write(28,*)'sm:',sm,' taum:',taum
               write(28,*)'Program stop'
               stop

2000           continue
               if(drai(kf).lt.0.)drai(kf)=0.
            enddo

            do kf=1,nfn-1
               drdfb=(drai(kf)-drai(kf-1))/(fnt(kf)-fnt(kf-1))
               drdft=(drai(kf+1)-drai(kf))/(fnt(kf+1)-fnt(kf))
               drdf(kf) = (drdfb+drdft)/2.
            enddo

            do kf=1,nfmean
               xi(kf) = fnt(kf)
               yi(kf) = drdf(kf)
            enddo 
            call polfit(xi,yi,nfmean,aj,nde,err,cj,sl)

            do kf=nfmean,nfn-1
               xi(kf-nfmean+1) = fnt(kf)
               yi(kf-nfmean+1) = drdf(kf)
            enddo 
            call polfit(xi,yi,nfmean,bj,nde,err,cj,sl)

            ajdr(0,ks0) = drai(nfmean)
            bjdr(0,ks0) = drai(nfmean)
            do j=1,nde+1 
               ajdr(j,ks0) = aj(j-1)/float(j)
               bjdr(j,ks0) = bj(j-1)/float(j)
            enddo

         enddo

      return
      end


c=======================================================================
c***********************************************************************
      subroutine tau(aa,bb,fnet,rdep,drk0,X0,XN,TAUI,NI)
c***********************************************************************
c=======================================================================

      implicit none
c      implicit double precision (a-h,o-z)
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer, intent(in) :: ni
      integer i,k,li

      real*8, intent(in) :: aa,bb,fnet,rdep,drk0,X0,XN
      real*8, intent(out) :: TAUI
      
      real*8  den,fac,fx,h,sum,x
c
c-----------------------------------------------
      LI=2*NI
      H=(XN-X0)/FLOAT(LI)
      SUM=0.
      DO I=0,LI
         K=2*INT((FLOAT(I)+0.01)/2.)
         IF(K.NE.I)THEN
            FAC=4.
         ELSEIF((I.EQ.0).OR.(I.EQ.LI))THEN
            FAC=1.
         ELSE
            FAC=2.
         ENDIF
         X=X0+FLOAT(I)*H
         den = fnet-drk0*dexp(aa+bb/x)
         if(dabs(den).gt.1.d-10)then
            fx = rdep/den
         else
            fx = rdep/1.d-10
         endif 
         SUM=SUM+FAC*FX
      enddo  
      TAUI=SUM*H/3.
      TAUI=dabs(TAUI)
      RETURN
      END


c=======================================================================
c***********************************************************************
      subroutine polfit(xi,yi,ni,aj,ndeg,err,cj,sl)
c***********************************************************************
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer, intent(in) :: ni,ndeg
      integer i,j,k,nc,ne
      
      real*8  differ,sum,xij,xik,yical,aj,err,sl,xi,yi,cj
c
c-----------------------------------------------
      
      dimension xi(ni),yi(ni),aj(0:ndeg),cj(ndeg+1),sl(ndeg+1,ndeg+2)
      ne=ndeg+1
      nc=ndeg+2
      do 100 k=1,ne
         sum=0.
         do 200 i=1,ni
            call puiss(xi(i),k-1,xik)
            sum=sum+xik*yi(i)
 200     continue
      sl(k,nc)=sum
 100  continue
      do 300 k=1,ne
         do 400 j=1,ne
            sum=0.
            do 500 i=1,ni
               call puiss(xi(i),k-1,xik)
               call puiss(xi(i),j-1,xij)
               sum=sum+xik*xij
 500        continue
         sl(k,j)=sum
 400     continue
 300  continue
      call gauss(ne,sl,cj)
      do 600 j=0,ndeg
         aj(j)=cj(j+1)
 600  continue
      err=0.
      do 700 i=1,ni
         yical=aj(0)
         do 800 j=1,ndeg
            call puiss(xi(i),j,xij)
            yical=yical+aj(j)*xij
 800     continue
      differ=yical-yi(i)
      err=err+differ*differ
 700  continue
      return
      end

c=======================================================================
c***********************************************************************
      subroutine puiss(x,k,xk)
c***********************************************************************
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none  
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer, intent(in) :: k
      integer :: i
      
      real*8, intent(in)  :: x
      real*8, intent(out) :: xk
c
c-----------------------------------------------
    
      xk=1.
      if(k.le.0)goto 200
      do 100 i=1,k
         xk=xk*x
 100  continue
      goto 400
 200  if(k.eq.0)goto 400
      do 300 i=1,-k
         xk=xk/x
 300  continue
 400  continue
      return
      end


c=======================================================================
c***********************************************************************
      subroutine gauss(ne,sl,x)
c***********************************************************************
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none        
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer, intent(in) :: ne
      integer jc,je,jr,nc
      
      real*8  coeff1,diag,rsum,sl,x
c
c-----------------------------------------------
      
      dimension sl(ne,ne+1),x(ne)
c=======================================================================
c   Subroutine Gauss solves a system of simultaneous linear algebraic
c   equations by Gaussian elimination and back substitution.
c   The number of equations (equal to the number of unknowns) is ne. 
c   The coefficients are in array sl(ne,ne+1), where the last 
c   column is the constants on the right sides of the equations. 
c   The answers are returned in the array x(ne).
c=======================================================================
      nc=ne+1
      do 100 je=1,ne
         diag=sl(je,je)
         if(diag.eq.0.)call exch(ne,nc,je,sl,diag)
         do 200 jc=je,nc
            sl(je,jc)=sl(je,jc)/diag
 200     continue
         do 300 jr=je+1,ne
            coeff1=sl(jr,je)
            do 400 jc=je,nc
               sl(jr,jc)=sl(jr,jc)-sl(je,jc)*coeff1
 400        continue
 300     continue
 100  continue
      x(ne)=sl(ne,nc)
      do 500 je=ne-1,1,-1
         rsum=0.
         do 600 jc=je+1,nc-1
            rsum=rsum+x(jc)*sl(je,jc)
 600     continue
         x(je)=sl(je,nc)-rsum
 500  continue
      return
      end
      
c############################################################################

c=======================================================================
c***********************************************************************
      subroutine init_c (y,ngt)
c***********************************************************************
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/climin.common'
      include './com_18/coord.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/eco.common'
      include './com_18/inidata.common'
      include './com_18/monwat.common'
      include './com_18/varday.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer idn,j,ngt
      real*8  y
c
c-----------------------------------------------
      
      dimension y(nequat)

c=====================================================================72
c     Initial values for every grid points and hydrological reservoirs 
c=====================================================================72
         do j=1,nres
          y(j) = ywat_ini(j,ngt)
         end do
         if (y(1).lt.0.) y(1)=0.
         if (y(2).lt.wpi) y(2)=wpi
         if (y(2).gt.fsi) y(2)=fsi

         do idn=1,nd
            aswday(idn)= (y(2)-wpi)/(fci-wpi)
         enddo

         aswmin = 999.

      return
      end

c###########################################################################

c=======================================================================
c***********************************************************************
      subroutine ode(tbegin,tend,y,stept,niter,ipr)
c***********************************************************************
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/coldrv.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,ipr,iter,j,niter
      real*8  drv,time,y,ynew
c
c-----------------------------------------------      
      real*4 tbegin,tend,stept
      dimension y(nequat), ynew(nequat), drv(nequat)

c   reading of the control parameters :
c   stept  : integration step (constant during the whole simulation)
c   niter  : maximum number of correction iterations
c   tbegin : initial time
c   tend   : final   time
c   ipr    : unite pour les impressions
c ----------------------------------------------------------------------

c initial conditions
c ----------------------------------------------------------------------
      time = tbegin
      call timdep(stept,time)


c the integration is performed with "runge-kutta" for the first
c "norder" steps :
c derivatives "drv" are computed at the current time value (givedrv)
c and kept in "tbldrv" whose back differences will be computed in
c "initbdo".
c "runkut4" computes "y" for the next time value.
c ----------------------------------------------------------------------

      do j = norder,1,-1
         call givedrv(stept,time,y,drv)
         call check(stept,time,y,drv)
         call printing(ipr,time,y,drv)
         do i = 1, nequat
            tbldrv(i,j) = drv(i)
         enddo
         call runkut4(stept,time,y,drv)
         time = time + stept
         call timdep(stept,time)
      enddo
      call initbdo

c   prediction-correction scheme begins :
c ----------------------------------------------------------------------
9990  continue
      call givedrv(stept,time,y,drv)
      call check(stept,time,y,drv)
      call printing(ipr,time,y,drv)
      call backdiff(drv)
      call bashfor(stept,drv,y,ynew)     
      time = time + stept
      call timdep(stept,time)
      call remember(drv)
      do iter = 1, niter
         call givedrv(stept,time,ynew,drv)
         call backdiff(drv)
         call moulton(stept,drv,y,ynew)
      enddo
      do i = 1, nequat
         y(i) = ynew(i)

      enddo
      if(time .lt. tend) goto 9990

CVC      call givedrv(stept,time,y,drv)
CVC      call check(stept,time,y,drv)
      call printing(ipr,time,y,drv)

      return
      end

c=======================================================================
c***********************************************************************
      subroutine timdep(stept,time)
c***********************************************************************
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none

      include './com_18/parameter.common'
      include './com_18/climin0.common'
      include './com_18/climin.common'
      include './com_18/coord.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/eco.common'
      include './com_18/nspc.common'
      include './com_18/solpar.common'
      include './com_18/varnow.common'
      include './com_18/varday.common'
      include './com_18/vegfr.common'
      include './com_18/xlaic.common'
      real*4, intent(in) :: stept

c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer idn,idn1,idn2,ih,ip,j,jh,ngf,ngf2,ny0
      
      real*8  dn,strfac,tcor,tem1,tem2,time,tmin10,tnmin1,tnmin2,wacor
     &      ,wat10
c
c-----------------------------------------------


      ny0 = int(time/float(nd)+0.000001)+1
      dn  = time - float(ny0-1) * float(nd)
      idn=int(dn+1.00001)

         preci=prc(idn)
         temi=tcel(idn)
         delti=tdiff(idn)
         shi=sunhour(idn)
         hai=rhu(idn)
         windi=win(idn)

         do ih=0,nh2
            solg(ih) = solday(idn,ih)
            ftrs(ih) = ftrday(idn,ih)
            xmu(ih) = xmuday(idn,ih)
         enddo

         do ih=nh2+1,nh
            jh = nh+1-ih
            solg(ih) = solday(idn,jh)
            ftrs(ih) = ftrday(idn,jh)
            xmu(ih) = xmuday(idn,jh)          
         enddo 


c   tmin10: average min. temp. of the last 2*ngf2 days 
c   wat10 : average soil water of the last 2*ngf2 days 
c   wat10 : average soil water of the last 2*ngf2 days 
c   tem1  : average temp. of the last period of ngf2 days 
c   tem2  : average temp. of the last but one period of ngf2 days 
c ----------------------------------------------------------------------

         tem1 = 0.
         tem2 = 0.
         tmin10 = 0.
         wat10 = 0.
         ngf2 = 7
         ngf = 2*ngf2
         do j = 1,ngf2
            idn1 = idn-j
            if (idn1.le.0) idn1=idn1+nd
            idn2 = idn1-ngf2
            if (idn2.le.0) idn2=idn2+nd
            tem1 = tem1+tcel(idn1)/ngf2
            tem2 = tem2+tcel(idn2)/ngf2
            tnmin1=tcel(idn1)-tdiff(idn1)/2.
            tnmin2=tcel(idn2)-tdiff(idn2)/2.
            tmin10 = tmin10 + (tnmin1+tnmin2)/ngf
            wat10 = wat10 + (aswday(idn1)+aswday(idn2))/ngf
         enddo

c   grflx : heat flux into the ground H [W m-2] 
c   (Mintz and Walker, J. Appl. Met., 32, p. 1313, 1993) 
c ----------------------------------------------------------------------

       grflx = 48.426 * (tem1-tem2) / (ngf2*stept)

c   plai(ip) : lai of plant type ip over grid cell igr
c  [m2/m2] 
c ----------------------------------------------------------------------

         wacor = wat10/0.25
         if (wacor.ge.1.)wacor=1.
         if (wacor.le.0.)wacor=0. 

         do ip=1,npft
            tcor = (tmin10-t1pft(ip))/(t2pft(ip)-t1pft(ip))
            if (tcor.ge.1.)tcor=1.
            if (tcor.le.0.)tcor=0.
            strfac = min(wacor,tcor)
            plai(ip)=xlmin(ip)+strfac*(xlmax(ip)-xlmin(ip))
         enddo

      return
      end
c=======================================================================
c***********************************************************************
      subroutine givedrv(stept,time,y,drv)
c***********************************************************************

c=======================================================================
c the system to be solved consists in "nequat" ordinary differential
c equations of the form
c      .
c      y(1)      = drv(1)
c      .
c      y(2)      = drv(2)
c                .
c                .
c                .
c      .
c      y(nequat) = drv(nequat)
c
c where drv(i) depends on y(1), y(2), ..., y(nequat), and on time.
c please give your vector "drv".
c for example :
c
c     --------------------------------------
c      ny0 : year number
c      dn  : day number
c     --------------------------------------
c      nr=1 : snow (H2O)
c      nr=2 : soil water (H2O)
c     --------------------------------------
c     --------------------------------------
c      y(nres)   : reservoirs (gC m-2) or (gN m-2) or mm H2O
c      drv(nres) : dy/dt (gC m-2 day-1) or (gN m-2 day-1)
c                            or mm H2O day-1
c     --------------------------------------
c      fin_w(nres)  : input flux of each reservoir (mm H2O day-1)
c      fout_w(nres) : output flux of each reservoir (mm H2O day-1)
c     --------------------------------------
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/climin0.common'
      include './com_18/cloud.common'
      include './com_18/coord.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/drain.common'
      include './com_18/eco.common'
      include './com_18/ecoin.common'
      include './com_18/ecopro.common'
      include './com_18/envi.common'
      include './com_18/fgr.common'
      include './com_18/flux_w.common'
      include './com_18/frac_change.common'
      include './com_18/landuse.common'
      include './com_18/nspc.common'
      include './com_18/radcst.common'
      include './com_18/smrd.common'
      include './com_18/solpar.common'
      include './com_18/temper.common'
      include './com_18/varnow.common'
      include './com_18/vegfr.common'
      include './com_18/waflux.common'


c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer imeth,ip,j,jbrac,jnum,k0w,k1w,k2w,kj,kw,nint,nitmax,nr
     &      ,num1,numc,numit,ny0
     
      real*8  airmw,albs,albsn,alnlw,alnsw,avh2o,bts,cpair,delta,dn,dr
     &      ,drv,eint,eintpl,ej,ej1,ej2,emia,emidry,emis,emisn,emiwet
     &      ,ermax,err,err1,err2,esnmax,esnow,esoil,esomax,fgr_crop
     &      ,fgr_herb,fgr_past,fgr_tree,fh2o,firdown,fnet,fnmax,fnsw
     &      ,fntj,frcso,frctr,fsnow,gama,pcdew,pcrain,prod,qfrac,qh2o,qx
     &      ,ra,rair,ram,rapso,rblws,rblwv,rbw,rbwm,rhoair,rnfac,smlmax
     &      ,solnet,t0dr,tatm,tca,tcb,time,tm3,tmfac,tr,tsf,tsf1,tsf2
     &      ,tsurf,us,usm,usmulm,usmulv,uss,was,x,x0,xii,xk,xle,xleadi
     &      ,xlice,xlog2,xlsml,xlw,xnn,xr,xrblwm,y,z0,zzlogm
c
c-----------------------------------------------

      real*4 tc,ftc,psat
      real*4 stept
      real*4 tmax,tmin
      real*4 cond_max,exces_run
      real*4 plant_lai,plant_fgrn

      dimension y(nequat),drv(nequat),ej(0:nde+1)

      ny0 = int(time/float(nd)+0.000001)+1
      dn  = time - float(ny0-1) * float(nd)

         asw = (y(2)-wpi)/(fci-wpi)

c   relative water amount with respect to saturation 
c   (used in emissivity and drainage calculations)
c-----------------------------------------------------------------------
         was = (y(2)-wpi)/(fsi-wpi)
         if(was .lt. 0.) was = 0.
         if(was .gt. 1.) was = 1.

c=======================================================================
c      hydrological model
c=======================================================================

c   bts: beta_Ts (Mintz and Walker, p. 1326)
c-----------------------------------------------------------------------
c        bts = 1. - dexp(-6.8*asw)
         bts = asw
         if(bts.lt.0.)bts=0.
         if(bts.gt.1.)bts=1.

c   diurnal tmin and tmax 
c-----------------------------------------------------------------------
         if(idayt .eq. 1) then
            tmax =  temi + 0.5 * delti
            tmin =  temi - 0.5 * delti
         elseif(idayt .eq. 2) then
            tmax =  temi
            tmin =  temi
         endif

c   tk : surface air temperature in Kelvin   
c-----------------------------------------------------------------------
         tk=temi+temp0

c   snd   : snow depth in mm 
c-----------------------------------------------------------------------
         snd = 10. * y(1)
         if(snd.lt.0.)snd=0.

c   fsnow : snow cover
c-----------------------------------------------------------------------
         fsnow = sqrt(0.01*snd)
         if(fsnow.gt.1.)fsnow=1.

c   fgreen: green cover  
c-----------------------------------------------------------------------
c         tlai = 0.
c         fgreen = 0.

         do ip =1,npft
c            dlai = plai(ip) / 5000.
c            prob=dexp(5000.*dlog(1.-dlai))
c            fgrn(ip) = (1.-prob)
            plant_lai = plai(ip)
            call fgrn_calc(plant_lai,plant_fgrn)
            fgrn(ip) = plant_fgrn 
            
c            fgreen = fgreen+frc(ip)*fgrn(ip)
         enddo
c         tlai = 5000.*(1.-dexp( (dlog(1.-fgreen))/5000. ))

         fgr_tree = 0.
         if(ntree.gt.0) then
          do ip = nherb+nbush+1, npft0
           fgr_tree = fgr_tree + fgrn(ip)*frac(ip)
          end do
         endif

         fgr_herb = 0.
         if(nherb+nbush.gt.0) then
          do ip = 1, nherb+nbush 
           fgr_herb = fgr_herb + fgrn(ip)*frac(ip)
          end do        
         endif

         fgr_crop = 0.
         if(ncrop.gt.0) then
          do ip = npft0+1, npft0+ncrop
            fgr_crop = fgr_crop + fgrn(ip)*frac(ip)
          end do
         endif
	 
         fgr_past = 0.
         if(npast.gt.0) then
          do ip = npft0+ncrop+1, npft
            fgr_past = fgr_past + fgrn(ip)*frac(ip)
          end do
         endif	 

c         fgreen = fgr_tree + (1.-fgr_tree)*fgr_herb +fgr_crop +fgr_past
         fgreen = fgr_tree + (frac_nat(igr)-fgr_tree)
     &   *fgr_herb +fgr_crop +fgr_past	 
	 
c        tlai = -alog(1.-fgreen)/(0.5*clump_fac) 
         tlai = 0.
         do ip = 1, npft
          tlai = tlai + frc(ip)*plai(ip)
         end do

c   fgs: snow-free green cover  
c-----------------------------------------------------------------------
         fgs = fgreen * (1.-fsnow)
         fsn = fsnow

c=======================================================================
c   ALBEDO CALCULATION 
c=======================================================================

c   albs: soil albedo  
c-----------------------------------------------------------------------
         albs=1.5*(0.10+0.1*colour+0.07*(1.-y(2)/fsi))

c   albsn: snow albedo  
c-----------------------------------------------------------------------
         if (temi.lt.0.) then
            tmfac=0.1*(temi+10.)
            if(tmfac.lt.0.001) tmfac=0.001
            tm3 = tmfac*tmfac*tmfac
            alnsw = 0.85 - 0.20 * tm3
            alnlw = 0.65 - 0.16 * tm3
            albsn = 0.5*(alnsw+alnlw)
         else
            albsn = 0.57
         endif

c   alb: average (soil+veg+snow) albedo  
c-----------------------------------------------------------------------
         alb = fsnow*albsn
     1         + (1.-fsnow)*(albv*fgreen+albs*(1.-fgreen))


c=======================================================================
c   THERMODYNAMICAL QUANTITIES  
c   psat: pression de vapeur saturante de H2O [Pa]  
c   delta: s = d(psat)/dT  [Pa K-1]  
c   xlw: chaleur latente L d'evaporation de H2O [J kg-1]  
c=======================================================================
         call esat(tk,psat)
         if (tk.ge.temp0) then
            delta=(psat/tk)*(asat/tk-bsat)
            xlw=2.497d+6 - 2.31d+3*(tk-temp0)
         else
            delta=psat*eaice/(rgas*tk*tk)
            xlw=eaice/h2omw
         endif

c   eh2o: H2O vapour pressure (Pa)  
c   fh2o: volume mixing ratio of H2O  
c   qh2o: specific humidity (kg kg-1)  
c   airmw: air molecular weight [kg mol-1]  
c   rhoair: air density (perfect gas law) [kg m-3]  
c   rair: gas constant of wet air (J K-1 kg-1)
c-----------------------------------------------------------------------
         eh2o = hai*psat
         fh2o = eh2o/patm
         qh2o = epsi*fh2o
         airmw = (qh2o + (1.-fh2o)) * drymw
         rair = rgas/airmw
         rhoair = patm/(rair*tk)        
        
c   cpair: specific heat cp of air at constant p [J kg-1 K-1]  
c-----------------------------------------------------------------------
         cpair = cph2o*qh2o + cpdry*(1.-qh2o) 

c   gama: psychrometric constant [Pa K-1]  
c-----------------------------------------------------------------------
         gama = cpair*patm/(epsi*xlw)
         rscp_air = rair/cpair
        
c=======================================================================
c   EMISSIVITY CALCULATION 
c   emisn: emissivity of snow 
c   emiv: emissivity of vegetation 
c   emis: soil emissivity 
c   emisf: average surface emissivity 
c   emia: atmospheric emissivity 
c=======================================================================

c   snow 
c-----------------------------------------------------------------------
         emisn = 0.96

c   soil
c-----------------------------------------------------------------------
         emiwet = 0.01*(0.95*san+0.96*sil+0.97*cla)
         emidry = 0.01*(0.87*san+0.93*sil+0.95*cla)
         emis = emidry + was*(emiwet-emidry)

c   average emissivity of the surface
c-----------------------------------------------------------------------
         emisf = fsnow*emisn
     1         + (1.-fsnow)*(emiv*fgreen+emis*(1.-fgreen))

c   atmosphere  
c   Staley & Jurica    
c         aa = 0.67-0.04*(1.013d+5-patm)/0.303d+5
c         emia = aa*((eh2o/100.)**0.08)
c   Guyot
c         emia = 0.56+0.08*sqrt(eh2o/100.)
c   Brutsaert, 1975
c         emia = 1.24*((0.01*eh2o/tk)**0.142857)
c   Crawford & Duchon, 1999 (modified Brutsaert)
c         xmon=(dn+15.2)/30.416667
c         if (ylati.lt.0.) xmon = xmon+6. 
c         aa = 1.22+0.06*dsin((xmon+2.)*pi/6.)
c         emia = aa*((0.01*eh2o/tk)**0.142857)
c   Anderson 1954
c-----------------------------------------------------------------------
         emia = 0.68+0.036*sqrt(0.01*eh2o)

         if (emia.gt.1.) emia=1.

c=======================================================================
c   AERODYNAMIC RESISTANCE 
c   rblwm, rblcv: aerodynamic resistances [s m-1] 
c                 for water (averaged over vegetation and soil) 
c                 and carbon (for each vegetation type) 
c                 respectively
c=======================================================================

         if(windi.lt.0.1)windi=0.1
         uss = usmuls*windi 
         rblws = windi/(uss*uss) + 6.2/(uss**0.67)
         xrblwm = (1.-fveg)*dlog(rblws)
c mean of log(zzra-disd)/z0
         xlog2 = (1.-fveg)/(zzlogs*zzlogs)
         do ip = 1, npft
            z0 = z0vw(ip) + fgrn(ip) * (z0vs(ip)-z0vw(ip))
            zzlog(ip) = dlog((zzra(ip)-disd(ip))/z0)
            xlog2 = xlog2 + frc(ip)/(zzlog(ip)*zzlog(ip))
         end do
         if (xlog2.gt.1.d-10) then
            zzlogm = dsqrt(1./xlog2)
         else
            zzlogm = 1.
         endif
c mean aerodynamic and boundary layer resistances (for water)    
         usmulm = 0.40/zzlogm
         usm = usmulm*windi
         ram = windi/(usm*usm)
         rbwm = 6.2/(usm**0.67)
         rblwm = ram + rbwm
c aerodynamic and boundary layer resistances of each plant type (for carbon) 
         do ip = 1, npft
            usmulv = 0.40/zzlog(ip)
            us = usmulv*windi
            ra = windi/(us*us)
            rbw = 6.2/(us**0.67)
            rblwv = ra + rbw
            rblcv(ip) = (ra+1.37*rbw)
         enddo



c   snowmelt (sml), rainfall (rf), and snowfall (sf) 
c  (mm day-1)
c-----------------------------------------------------------------------
        if(tmax.le.0.)then
            sml = 0.
            rf = 0.
            sf = preci
         else
            if(tmin.ge.0.)then
               sml= 4.57 * temi
               rf = preci
               sf = 0.
            else
               if((tmax-tmin).le.1.d-10)then
                  pcrain = 0.5
               else
                  pcrain = tmax / (tmax-tmin)
               endif
               sml = 0.5*pcrain * 4.57*tmax
               rf = pcrain * preci
               sf = (1.-pcrain) * preci
            endif
            smlmax = y(1) / stept
            if(sml.gt.smlmax)sml=smlmax
            if(sml.lt.0.)sml=0.
         endif

c   xlsml: Energy used for snow melt (W m-2)
c   xlice: Latent heat of ice melt (J kg-1)
c-----------------------------------------------------------------------
         xlice = 3.34d+5
         xlsml = xlice*sml/86400.

c   input fluxes (fin) 
c   (mm day-1) 
c-----------------------------------------------------------------------
         fin_w(1) = sf
         fin_w(2) = rf + sml
         exces_run = 0.

         cond_max = (12./12.) * 240.* exp(acd+bcd*rootd/fsi)             
         if (fin_w(2).gt.cond_max) then
           exces_run = fin_w(2)-cond_max
           fin_w(2) = cond_max
         endif

c=======================================================================
c   POTENTIAL EVAPOTRANSPIRATION (PENMAN)
c        pet (mm day-1) 
c=======================================================================

c   factors for Penman relationship
c-----------------------------------------------------------------------

         xleadi=(rhoair*cpair/(delta+gama))*psat*(1.-hai)/rblwm
         rnfac = delta/(delta+gama)

c  tc = cloud (bottom) temperature (K)
c-----------------------------------------------------------------------
         tc = tk-11.
	 
         if (shi.le.0.999) then
 
          tcb = tk+5.
          tca = tk-85.
          nint=4
          ermax = 0.01
          nitmax =100
          imeth = 1	  
          call nonlineq(tca,tcb,nint,tc,ftc,ermax,nitmax,imeth,numc)

	  
	 endif
        
c  tatm = atmosphere (lower 1 km) temperature (K), i.e. average 
c         temperature of the atmospheric layer contributing to the 
c         downward infrared radiation during clear sky conditions 
c-----------------------------------------------------------------------

         tatm = tk
        
c   firdown = downward infrared radiation (W m-2)
c-----------------------------------------------------------------------
         firdown = emia*sigma*tatm*tatm*tatm*tatm
     1     +(1.-shi)*(1.-emia)*sigma*tc*tc*tc*tc

c   solnet : net solar radiation flux [W m-2]
c-----------------------------------------------------------------------
         solnet = solg(0) * (1.-alb)

c   loop for the calculation of Tsurf
c-----------------------------------------------------------------------
         tsurf = tk
         num1 = 10
         jbrac = 1
         numit = 0

         do jnum = 1,100
            numit = numit+1
            tsf = tsurf
c   rnet : net radiation flux Rn [W m-2] 
c-----------------------------------------------------------------------
            fird = firdown
            rnet = solnet + emisf*firdown 
     &              - emisf*sigma*tsf*tsf*tsf*tsf

c   xle : LE* from the surface (Penman) [W m-2]
c-----------------------------------------------------------------------
            xle = (rnet-grflx-xlsml)*rnfac + xleadi
c            if(xle.lt.0.)xle=0.
c   pet : potential evapotranspiration E* (Penman) [mm day-1] 
c-----------------------------------------------------------------------
            pet = 86400.*xle/xlw


c   eint : intercepted precipitation directly re-evaporated [mm day-1]
c   esnow: snow evaporation from soil-vegetation [mm day-1] 
c   esoil: evaporation from soil-vegetation [mm day-1]
c-----------------------------------------------------------------------
            if (pet.gt.0.) then
               eint=0.
               do ip=1,npft
                  if(fgrn(ip).eq.0.)then 
                     eintpl=0.
                  else
                     x=preci/pet
                     xii=(stept*pet*fgrn(ip)/(0.2*plai(ip))) 
                     x0=1./(0.84640+0.83842*xii)
                     xr=x0+1.
                     tr=0.7225-0.2529*exp(-xii/6.738)
                     xk=1./tr-1.
                     if(x.le.x0)then
                        qfrac=0.
                     else
                        if(x.le.xr)then
                           xnn=1.352
                        else
                           xnn=1.122
                        endif
                        qx=(x-x0)**xnn
                        qfrac=qx/(qx+xk)
                     endif
                     eintpl=preci*fgrn(ip)*(1.-qfrac)
                     if(eintpl.gt.pet)eintpl=pet
                  endif
                  eint=eint+eintpl*frc(ip)
               enddo
               esnow = fsnow * (pet-eint)
               esnmax = y(1) / stept - sml
               if (esnow.gt.esnmax) esnow = esnmax
               if(esnow.lt.0.)esnow=0.
               esoil = bts * (pet-eint-esnow)
               esomax = (y(2)-wpi) / stept
               if (esoil.gt.esomax) esoil = esomax
               if(esoil.lt.0.)esoil=0.
            else
               eint = 0.
               if(tmax.le.0.)then
                  esoil = 0.
                  esnow = pet
               elseif(tmin.ge.0.)then
                  esoil = pet
                  esnow = 0.
               else
                  if((tmax-tmin).le.1.d-10)then
                     pcdew = 0.5
                  else
                     pcdew = tmax / (tmax-tmin)
                  endif
                  esoil = pcdew * pet
                  esnow = (1.-pcdew) * pet
               endif
            endif


c   axle : (actual) latent heat LE from the surface [W m-2] 
c-----------------------------------------------------------------------
            axle = (eint+esnow+esoil) * xlw / 86400.
c            if(axle.lt.0.)axle=0.

c   xh : (actual) sensible heat H from the surface [W m-2] 
c-----------------------------------------------------------------------
            xh = rnet-grflx-xlsml-axle

            tsurf = tk + xh*rblwm/(rhoair*cpair)

            err = tsurf - tsf

            if (dabs(tsurf).ge.1.d+10) numit=num1
            if (dabs(err).lt.0.05) goto 340

            if (numit.eq.1) then
               tsf1=tsf
               err1=err
            elseif (numit.ge.num1) then

               if (jbrac.eq.1)then
                  if (numit.eq.num1) then
                     tsurf = tk + 1.0
                  else
                     tsf2 = tsf
                     err2 = err
                     prod = err1*err2
                     if (prod.gt.0.) then
                        if (dabs(err1).lt.dabs(err2)) then
                           tsurf = tsf1+(tsf1-tsf2)*1.7
                        else
                           tsurf = tsf2+(tsf2-tsf1)*1.7 
                           tsf1 = tsf2
                           err1 = err2
                        endif
                     else
                        tsurf = tsf1+(tsf2-tsf1)*dabs(err1)
     &                         /dabs(err1-err2)
                       jbrac = 0
                     endif  
                  endif
               else
                  prod = err*err2
                  if (prod.gt.0) then
                     tsf2 = tsf
                     err2 = err
                  else
                     tsf1 = tsf
                     err1 = err
                  endif
                  tsurf = tsf1+(tsf2-tsf1)*dabs(err1)/dabs(err1-err2)  
               endif

            endif
         enddo
         write(28,*)' CALCUL DE TS (SUB GIVEDRV)'
         write(28,*) ylongi,ylati,igr,ny0,dn,
     &     ' numit=',numit
         write(28,*)tsf1,err1
         write(28,*)tsf2,err2
         stop
340      continue
c   ts : average surface (snow+soil+vegetation) temperature (C) 
c-----------------------------------------------------------------------
         ts = tsurf - temp0

c   soil+vegetation actual evapotranspiration (soe) 
c   snow actual evaporation (sne) 
c   total actual evapotranspiration (aet) 
c   (mm day-1) 
c-----------------------------------------------------------------------
         if(preci.ne.0.)then
            rapso = rf/preci
         else
            rapso = 0.
         endif
         soe = eint * rapso + esoil
         sne = eint * (1.-rapso) + esnow
         aet = soe + sne
         svevp = esoil
	 
         
c   rtrans = ratio of plant transpiration to soil+vegetation evapotr.
         frctr = 1.-exp(-tlai)
         if (tlai.le.1.) then
           frcso = 1.-0.666*tlai
         else
           frcso = exp(-tlai)/1.1
         endif  
         rtrans = frctr/(frctr+frcso)
         if (rtrans.gt.1.) rtrans=1.
c old         if (rtrans.lt.0.6) rtrans=0.6
         if (rtrans.lt.0.) rtrans=0.


c   drun  : drainage (mm day-1) 
c   condw : soil conductivity (mm day-1) (Saxton et al, 1986)
c   thw   : soil water content (m3/m3)
c   ttl   : drainage characteristic time (day) 
c           if too short, re-evaluation of the drainage rate
c           to take the sharp variation of conductivity with
c           water content into account.
c   san, sil, cla: percent of sand, silt and clay
c-----------------------------------------------------------------------

          
c   interpolation sur le contenu en eau
c-----------------------------------------------------------------------

         k1w = 0
         if((k1w .lt. 1) .or. (k1w .ge. nwa)) k1w = 0
         if(was .lt. wa0(k1w)) k1w = 0
         k0w = k1w + 1
         do kw = k0w, nwa-1
            if(was .le. wa0(kw)) then
               k2w = kw
               goto 210
            endif
         enddo
         k2w = nwa
210      continue
         k1w =  k2w - 1

c   parametrisation interpolee pour dep. flux net
c-----------------------------------------------------------------------

         fnet = fin_w(2) - soe

         if(fnet.lt.0.)then
            if (dabs(fnet).ge.fntmax) fnet = -fntmax
            do j=0,nde+1
               ej1 = ajdr(j,k1w)
               ej2 = ajdr(j,k2w)
               ej(j) = ej1+(ej2-ej1)*(was-wa0(k1w))
     &                /(wa0(k2w)-wa0(k1w))
            enddo 
         else
            if (dabs(fnet).ge.fntmax) fnet = fntmax
            do j=0,nde+1
               ej1 = bjdr(j,k1w)
               ej2 = bjdr(j,k2w)
               ej(j) = ej1+(ej2-ej1)*(was-wa0(k1w))
     &                /(wa0(k2w)-wa0(k1w))
            enddo 
         endif
         dr = ej(0)
         do j = 1,nde+1
            fntj = 1.
            do kj = 1,j
               fntj = fntj*fnet
            enddo
            dr = dr + ej(j)*fntj
         enddo

         if (dr.lt.0.) dr=0.
         avh2o = fnet * stept + (y(2)-wpi)
         if (dr.gt.avh2o) dr=avh2o 
         drun = drn_fac * dr / stept

         t0dr = -2.
         if (tmax.le.t0dr) then
            drun = 0.
         elseif (tmax.lt.0.) then
            drun = (t0dr-tmax)*drun/t0dr
         endif


c   srun : surface runoff (mm day-1)
c-----------------------------------------------------------------------
         fnsw = fin_w(2) - soe - drun
         fnmax = (fsi-y(2)) / stept
         if (fnsw.gt.fnmax) then
            srun = exces_run + fnsw - fnmax
         else
            srun = exces_run
         endif

c   fout : output fluxes (mm day-1) 
c-----------------------------------------------------------------------
         fout_w(1) = sml + sne
         fout_w(2) = soe + drun + (srun - exces_run)


c=======================================================================
c       derivatives dy/dt [ mm day-1]
c=======================================================================

         do nr=1,nres
            drv(nr) = fin_w(nr) - fout_w(nr)
         enddo

      return
      end

c=======================================================================
c***********************************************************************
      subroutine esat (tk,es)
c***********************************************************************
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none

c=======================================================================
c   This subroutine calculates saturation vapour pressure of H2O.
c   INPUT :  tk = temperature [K]
c   OUTPUT : es = saturated vapour pressure of H2O [Pa]
c=======================================================================
      include './com_18/parameter.common'
      include './com_18/envi.common'
      include './com_18/temper.common'
      real*4, intent(in) :: tk
      real*4, intent(out) :: es

      if (tk.ge.temp0) then
         es = 100.*exp(-asat/tk-bsat*alog(tk)+csat)
      else
         es = psat0*exp(eaice*(tk-temp0)/(rgas*tk*temp0))
      endif

      return
      end

c=======================================================================
c***********************************************************************
      subroutine nonlineq(xa,xb,nint,x,fx,ermax,nitmax,imeth,num)
c***********************************************************************

c=======================================================================
c   Subroutine nonlineq solves a non linear equation of the type:
c           f(x) = 0
c   by the bisection method or the "regula falsi" method.
c   It calls the user-defined subroutine "funcx(x,fx)" which
c   defines the f(x) function. 
c
c      xa = lower bound of interval in which the solution is 
c           searched for
c      xb = upper bound of interval in which the solution is 
c           searched for
c      nint = number of intervals into which [xa,xb] is divided
c           when searching for a (smaller) interval of changing
c           sign (e.g. 10)
c      x = - at entry, x is set to any value between a and b;
c            it may represent an "initial guess" of the solution
c          - at exit, x contains the estimated value of the zero 
c            of the function 
c      fx = at output, fx contains f(x), i.e. fx should be close to
c           0, if x contains a good evaluation of the zero
c      ermax = maximum acceptable error on fx, 
c               i.e., when (abs(fx).le.ermax) the zero is accepted
c      nitmax = maximum number of iterations allowed (e.g., 100)
c      imeth = flag to select resolution method. Set imeth to 0 for
c              bisection method and to 1 for "regula falsi"
c      num = at exit, contains total number of iterations performed
c   
c   The variable "precmc" below must be set to a small number
c   representing machine precision. The relative error on the
c   zero (x) cannot be smaller than precmc. With single precision
c   a value of precmc of 1.e-6 is recommended. For double precision
c   a value as small as 1.d-12 can be used.
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      real*4 x,fx
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer imeth,k,nint,nit,nitmax,num
      real*8  delx,ermax,err,errx,fx1,fx2,precmc,prod,prod1,residu,x_old
     &      ,x1,x2,xa,xb
c
c-----------------------------------------------

      precmc = 1.d-10

      num = 0

      delx = (xb-xa)/float(nint)

c=======================================================================
c   initial try with previous value of x and values smaller or 
c   larger by an amount delx 
c   (sets values for x1 and x2 with fx1*fx2 < 0)
c=======================================================================

      call funcx(x, fx)

      err = abs(fx)
      if (err.le.ermax) goto 300

      x1 = x 
      fx1 = fx

      x = x1+delx
      if (x.gt.xb) x=xb
      if (x.lt.xa) x=xa

      num = num + 1
      call funcx(x, fx)

      prod = fx*fx1
      if (prod.gt.0) then
         x = x1-delx
         if (x.gt.xb) x=xb
         if (x.lt.xa) x=xa
         call funcx(x, fx)

         prod1 = fx*fx1
         if (prod1.le.0) then
            x2 = x
            fx2 = fx
            goto 200
         endif
      else
         x2 = x
         fx2 = fx
         goto 200
      endif

c=======================================================================
c   if initial try fails, search for the solution over the  
c   entire range xa to xb 
c=======================================================================

      x = xa

      num = num + 1
      call funcx(x, fx)

      x1 = x
      fx1 = fx

      do k = 1, nint
         x = xa+delx*float(k)
         num = num + 1

         call funcx(x, fx)

         prod = fx1*fx
         if (prod.gt.0.) then
            x1 = x 
            fx1 = fx 
         else
            x2 = x
            fx2 = fx
            goto 200
         endif 

      enddo

      write(*,*)'No solution found for x between xa and xb'
      write(*,*)' x=',x,' fx=',fx
      write(*,*)'program stop'
      stop

c=======================================================================
c   after finding two values of x with fx of different signs,
c   we solve the equation by bisection (imeth=0) or "regula falsi" 
c   (imeth=1) method  
c=======================================================================

 200  continue

c      write(*,*)'Interval of changing sign found'

      do nit = 1, nitmax

         num = num + 1

         x_old = x

         if (imeth.eq.0) then
            x = (x1+x2)/2.
         else
            residu = -fx1*(x2-x1)/(fx2-fx1)
            x = x1 + residu
         endif

         call funcx(x, fx)

         err=abs(fx)

         if(x.ne.0.) then 
            errx = abs((x-x_old)/x)
         else
            errx = abs(x-x_old)
         endif
         if ((err.le.ermax).or.(errx.le.precmc)) goto 300

         prod = fx*fx2
         if (prod.gt.0) then
           x2 = x
           fx2 = fx
         else
           x1 = x
           fx1 = fx
         endif

      enddo

      write(*,*)'Too many iterations in subroutine nonlineq'
      write(*,*)'num=',num,' x=',x
      write(*,*)'fx=',fx
      write(*,*)'program stop'
      stop

 300  continue


      return
      end



c=======================================================================
c***********************************************************************
      subroutine funcx(x, fx)
c***********************************************************************
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/cloud.common'
      real*4, intent(in) :: x
      real*4, intent(out) :: fx
      real*4  :: ecld
c=======================================================================
c   Subroutine funcx is designed by the user to define f(x).
c     x = at entry, contains the value of x at which f(x) is
c         evaluated; it cannot be changed by funcx
c     fx = at exit, fx contains the value of f(x)
c   Variables may be transferred from the main programme or
c   the subroutine calling "nonlineq" through user-defined commons. 
c=======================================================================

      call esat(x,ecld)
      fx = x - tk * ((ecld/eh2o)**rscp_air)

      return
      end


c=======================================================================
c***********************************************************************
      subroutine check(stept,time,y,drv)
c***********************************************************************
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none

      include './com_18/parameter.common'
      include './com_18/climin.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/eco.common'
      include './com_18/envi.common'
      include './com_18/flux_w.common'
      include './com_18/monthcst.common'      
      include './com_18/monwat.common'
      include './com_18/snow.common'
      include './com_18/temper.common'
      include './com_18/varday.common'
      include './com_18/varnow.common'
      include './com_18/waflux.common'      
      real*4 stept
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer idn,ip,m,nr,ny0
      real*8  dn,drv,time,y
c
c-----------------------------------------------
      
      dimension y(nequat),drv(nequat)

      ny0 = int(time/float(nd)+0.000001)+1
      dn  = time - float(ny0-1) * float(nd)
      idn=int(dn+1.00001)

         do nr =1,nres
           sumnet(nr) = sumnet(nr) + drv(nr)*stept
         enddo

         aswday(idn) = asw
         tladay(idn) = tlai

         water(idn) = asw
         snow_frac(idn) = fsn

         if (asw.lt.aswmin) aswmin=asw

c   rblday: carbon aerodynamic resistance (mol-1 m2 s)
c ----------------------------------------------------------------------
         do ip=1,10
            rblday(idn,ip) = 0.0244*((temi+temp0)/temp0)
     &                  *(patm0/patm)*rblcv(ip)
         enddo
         
c        sets monthly variables needed by CARAIB         
         if (idn.eq.1) then
           do m = 1,nm
             swcar(m) = 0.
             rtrcar(m) = 0.
             fsncar(m) = 0.
             svecar(m) = 0.
           end do
         endif
         m = imonth(idn)
         swcar(m) = swcar(m) + asw / float(mlength(m))
         rtrcar(m) = rtrcar(m) + max(rtrans,0.6d0) / float(mlength(m))
         fsncar(m) = fsncar(m) + fsn / float(mlength(m))
         svecar(m) = svecar(m) + svevp     
         

      return
      end

c=======================================================================
c***********************************************************************
      subroutine printing(ipr,time,y,drv)
c***********************************************************************
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none

      include './com_18/parameter.common'
      include './com_18/coord.common'
      include './com_18/cstpi.common'
      include './com_18/cte.common'
      include './com_18/eco.common'
      include './com_18/files_ibm.common'
      include './com_18/flux_w.common'
      include './com_18/iprt.common'
      include './com_18/monthcst.common'      
      include './com_18/monwat.common'
      include './com_18/nspc.common'
      include './com_18/prt.common'
      include './com_18/prt_ctrl.common'
      include './com_18/smrd.common'
      include './com_18/varnow.common'
      include './com_18/vegfr.common'
      include './com_18/waflux.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer iday,idnn,ip,ipr,k,km,m,nr,ny0
      real*8  dn,drv,eiry,time,wbudy,y
c
c-----------------------------------------------

      integer npint,ijour
      character*10 mois(nm)
      character*100 formatw
      dimension y(nequat),drv(nequat)
     

      data mois /'JANUARY','FEBRUARY','MARCH','APRIL','MAY','JUNE'
     1    ,'JULY','AUGUST','SEPTEMBER','OCTOBER','NOVEMBER','DECEMBER'/


      ny0 = int(time/float(nd)+0.000001)+1
      dn  = time - float(ny0-1) * float(nd)

      if (idaily_out.eq.0) then
        npint=nm
        do m = 1, nm
          mdur(m) = mlength(m)
        end do
      else
        npint=nd
        do iday = 1, nd
          mdur(iday)=1
        end do
      endif

c begins operations performed once a year (January 1st)
      if (ny0.ne.ny0prv) then


         open(29,file=filtim)
         write(29,*)'igr:',igr
         write(29,*)'ny0:',ny0,'    nyear:',nyear
         close(29)


         nprt=nstprt-1

         if (nsyr.gt.0) then

               prcy = float(nd)*prcy/nsyr
               runy = float(nd)*runy/nsyr
               soey = float(nd)*soey/nsyr
               svey = float(nd)*svey/nsyr
               pety = float(nd)*pety/nsyr
               sney = float(nd)*sney/nsyr

               do k=1,npint

                  swmth(k)  = swmth(k) / nsmth(k)
		  swmmmth(k)  = swmmmth(k) / nsmth(k)
                  rtrmth(k) = rtrmth(k) / nsmth(k)
                  petmth(k) = petmth(k) * mdur(k) / nsmth(k)
                  aetmth(k) = aetmth(k) * mdur(k) / nsmth(k)
                  runmth(k) = runmth(k) * mdur(k) / nsmth(k)
		  srunmth(k) = srunmth(k) * mdur(k) / nsmth(k)
                  drnmth(k) = drnmth(k) * mdur(k) / nsmth(k)
                  svemth(k) = svemth(k) * mdur(k) / nsmth(k)
		  eintmth(k) = eintmth(k) * mdur(k) / nsmth(k)
		  etrmth(k) = etrmth(k) * mdur(k) / nsmth(k)
		  esomth(k) = esomth(k) * mdur(k) / nsmth(k)
                  rblmth(k) = rblmth(k) / nsmth(k)
                  albmth(k) = albmth(k) / nsmth(k)
                  emimth(k) = emimth(k) / nsmth(k)
                  rnmth(k)  = rnmth(k) / nsmth(k)
                  firdmth(k) = firdmth(k) / nsmth(k)
                  xhmth(k)  = xhmth(k) / nsmth(k)
                  xlemth(k) = xlemth(k) / nsmth(k)
                  solmth(k) = solmth(k) / nsmth(k)
                  sfmth(k)  = sfmth(k) * mdur(k) / nsmth(k)
                  snemth(k) = snemth(k) * mdur(k) / nsmth(k)
                  smlmth(k) = smlmth(k) * mdur(k) / nsmth(k)
                  grfmth(k) = grfmth(k) / nsmth(k)
                  tsmth(k)  = tsmth(k) / nsmth(k)
                  fgsmth(k) = fgsmth(k) / nsmth(k)
                  fsnmth(k) = fsnmth(k) / nsmth(k)
		  sndmth(k) = sndmth(k) / nsmth(k)
                  tlamth(k) = tlamth(k) / nsmth(k)
               enddo 

         endif

         if (ny0.eq.ny0max+1) then
             if (iyprt.ge.1) then 
	     
	       

               if(iprt_sw.eq.1) 
     &            write(31,131) ylongi,ylati,(swmth(k),k=1,npint)
               if(iprt_swmm.eq.1) 
     &            write(531,631) ylongi,ylati,(swmmmth(k),k=1,npint)     
               if(iprt_rtr.eq.1) 
     &            write(532,131) ylongi,ylati,(rtrmth(k),k=1,npint)     
               if(iprt_pet.eq.1) 
     &            write(32,401) ylongi,ylati,(petmth(k),k=1,npint)
               if(iprt_aet.eq.1) 
     &            write(33,401) ylongi,ylati,(aetmth(k),k=1,npint)
               if(iprt_run.eq.1) 
     &            write(34,401) ylongi,ylati,(runmth(k),k=1,npint)
               if(iprt_fsn.eq.1) 
     &            write(35,131) ylongi,ylati,(fsnmth(k),k=1,npint)
               if(iprt_snd.eq.1) 
     &            write(535,135) ylongi,ylati,(sndmth(k),k=1,npint)
               if(iprt_srun.eq.1) 
     &            write(536,401) ylongi,ylati,(srunmth(k),k=1,npint)          
               if(iprt_drn.eq.1) 
     &            write(36,401) ylongi,ylati,(drnmth(k),k=1,npint)
               if(iprt_sve.eq.1) 
     &            write(37,401) ylongi,ylati,(svemth(k),k=1,npint)
               if(iprt_eint.eq.1) 
     &            write(537,401) ylongi,ylati,(eintmth(k),k=1,npint) 
               if(iprt_etr.eq.1) 
     &            write(538,401) ylongi,ylati,(etrmth(k),k=1,npint)
               if(iprt_eso.eq.1) 
     &            write(539,401) ylongi,ylati,(esomth(k),k=1,npint)              
               if(iprt_rbl.eq.1) 
     &            write(38,401) ylongi,ylati,(rblmth(k),k=1,npint)
               if(iprt_alb.eq.1) 
     &            write(39,131) ylongi,ylati,(albmth(k),k=1,npint)
               if(iprt_rn.eq.1) 
     &            write(40,401) ylongi,ylati,(rnmth(k),k=1,npint)
               if(iprt_grf.eq.1) 
     &            write(41,401) ylongi,ylati,(grfmth(k),k=1,npint)
               if(iprt_ts.eq.1) 
     &            write(42,132) ylongi,ylati,(tsmth(k),k=1,npint)
               if(iprt_fgs.eq.1) 
     &            write(43,131) ylongi,ylati,(fgsmth(k),k=1,npint)
               if(iprt_lai.eq.1) 
     &            write(44,1200) ylongi,ylati,(tlamth(k),k=1,npint)
               if(iprt_fird.eq.1) 
     &            write(45,401) ylongi,ylati,(firdmth(k),k=1,npint)
               if(iprt_xh.eq.1) 
     &            write(46,401) ylongi,ylati,(xhmth(k),k=1,npint)
               if(iprt_xle.eq.1) 
     &            write(47,401) ylongi,ylati,(xlemth(k),k=1,npint)
               if(iprt_sol.eq.1) 
     &            write(48,401) ylongi,ylati,(solmth(k),k=1,npint)
               if(iprt_sf.eq.1) 
     &            write(49,401) ylongi,ylati,(sfmth(k),k=1,npint)
               if(iprt_sne.eq.1) 
     &            write(50,401) ylongi,ylati,(snemth(k),k=1,npint)
               if(iprt_sml.eq.1) 
     &            write(51,401) ylongi,ylati,(smlmth(k),k=1,npint)
               if(iprt_emi.eq.1) 
     &            write(52,131) ylongi,ylati,(emimth(k),k=1,npint)


c            eiry = interception of rainfall by vegetation
               eiry=soey-svey   
               wbudy = prcy-runy-soey-sney

               if(iprt_yr.eq.1) write(30,125)igr,ylongi,ylati,
     1                fsi,fci,wpi,rootd,prcy,runy
     2                ,svey,eiry,sney,pety,wbudy,aswmin,fveg
               if(iprt_frc.eq.1) then
	          write(formatw,*)'(f8.3,1x,f8.3,',npft,'(1x,f12.8))'
	          write(530,formatw),ylongi,ylati,(frc(ip),ip=1,npft)
               endif
             endif

         endif

         do nr = 1,nres
            sumnet(nr) = 0.
         enddo

         nsyr = 0
         do k=1,npint
            nsmth(k) = 0
         enddo
        
         prcy=0.
         runy=0.
         soey=0.
         svey=0.
         pety=0.
         sney=0.
         do k=1,npint
            swmth(k) = 0.
	    swmmmth(k) = 0.
            rtrmth(k) = 0.
            petmth(k) = 0.
            aetmth(k) = 0.
            runmth(k) = 0.
	    srunmth(k) = 0.
            drnmth(k) = 0.
            svemth(k) = 0.
	    eintmth(k) = 0.
	    etrmth(k) = 0.
	    esomth(k) = 0.
            rblmth(k) = 0.
            albmth(k) = 0.
            emimth(k) = 0.
            rnmth(k) = 0.
            grfmth(k) = 0.
            tsmth(k) = 0.
            fgsmth(k) = 0.
            fsnmth(k) = 0.
	    sndmth(k) = 0.
            tlamth(k) = 0.
            firdmth(k) = 0.
            xhmth(k) = 0.
            xlemth(k) = 0.
            solmth(k) = 0.
            sfmth(k) = 0.
            snemth(k) = 0.
            smlmth(k) = 0.
         enddo

      endif

c begins operations performed every time step
      ny0prv = ny0
      nprt = nprt + 1

      idnn = int(dn+1.00001)
      km=0
 500  km=km+1
      if(idnn.gt.mdur(km))then
         idnn=idnn-mdur(km)
         goto 500
      endif

      nsyr = nsyr+1
      nsmth(km) = nsmth(km)+1

      prcy = prcy + preci
      runy = runy + drun+srun
      soey = soey + soe
      svey = svey + svevp
      pety = pety + pet
      sney = sney + sne
       
      swmth(km) = swmth(km) + asw
      swmmmth(km) = swmmmth(km) + (wpi + asw*(fci-wpi))      
      rtrmth(km) = rtrmth(km) + rtrans      
      petmth(km) = petmth(km) + pet
      aetmth(km) = aetmth(km) + aet
      runmth(km) = runmth(km) + drun+srun
      srunmth(km) = srunmth(km) + srun
      drnmth(km) = drnmth(km) + drun
      svemth(km) = svemth(km) + svevp
      eintmth(km) = eintmth(km) + (aet-svevp-sne)
      etrmth(km) = etrmth(km) + rtrans*svevp
      esomth(km) = esomth(km) + (1.-rtrans)*svevp
      rblmth(km) = rblmth(km) + rblwm
      albmth(km) = albmth(km) + alb
      emimth(km) = emimth(km) + emisf
      rnmth(km) = rnmth(km) + rnet
      grfmth(km) = grfmth(km) + grflx
      tsmth(km) = tsmth(km) + ts
      fgsmth(km) = fgsmth(km) + fgs
      fsnmth(km) = fsnmth(km) + fsn
      sndmth(km) = sndmth(km) + snd
      tlamth(km) = tlamth(km) + tlai
      firdmth(km) = firdmth(km) + fird
      xhmth(km) = xhmth(km) + xh
      xlemth(km) = xlemth(km) + axle
      solmth(km) = solmth(km) + solg(0)
      sfmth(km) = sfmth(km) + sf
      snemth(km) = snemth(km) + sne
      smlmth(km) = smlmth(km) + sml      
        

c begins full printing if ifull eq 1
c operations are performed every nstprt time steps, but only over 
c the last ny0prt years

      if (ifull.eq.1) then
         if ((ny0.ge.(ny0max-ny0prt+1)).and.(nprt.eq.nstprt)) then
            nprt = 0
            write(ipr,100) ny0,dn,mois(km),idnn

               write(ipr,1101) igr
               write(ipr,101) ylongi,ylati,temi
     &                       ,delti,preci,asw,solg(0)
               write(ipr,102) (y(k),k=1,nres)
               write(ipr,102) (drv(k),k=1,nres)
               write(ipr,102) (fin_w(k),k=1,nres)
               write(ipr,102) (fout_w(k),k=1,nres)
               write(ipr,105) fsi,fci,wpi,pet,soe
     &                       ,drun,srun,sne
               write(ipr,106) ts,hai,windi,tlai,alb
     &                 ,rnet,grflx,rblwm

         endif
      endif

 1101 format(1x,'STATION No',i6)
  100 format(1x,'YEAR=',I4,'   DAY NUMBER=',F6.2,6X,A10,I2)
  101 format(1x,f6.1,1x,f6.1,1x,f5.1,1x,f6.2,1x,f7.3,1x,f7.4,1x,f7.2)
  102 format(2(1x,d12.5))
  105 format(1x,3(1x,f7.2),5(1x,f7.3),2(1x,i3))
  106 format(1x,f6.2,1x,f6.3,2(1x,f5.2),1x,f6.3,3(1x,f6.1))
  115 format(1x,i4,1x,f5.1,1x,f6.1,f7.1,2(1x,i3))
  125 format(i6,2(1x,f8.3),4(1x,f7.2),2(1x,f8.2),4(1x,f7.2)
     &      ,1x,f8.2,2(1x,f6.3))
  131 format(2(1x,f8.3),366(1x,f7.4))
  631 format(2(1x,f8.3),366(1x,f8.2))
  132 format(2(1x,f8.3),366(1x,f6.2))
  135 format(2(1x,f8.3),366(1x,f9.0))  
  401 format(2(1x,f8.3),366(1x,1pe10.3))
 1200 format(2(1x,f8.3),366(1x,f4.1))

      return
      end


c=======================================================================
c***********************************************************************
      subroutine runkut4(stept,time,y,drv)
c***********************************************************************
c=======================================================================
c
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/rk.common'
      real*4 stept,stept2

c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i
      real*8  drv,time,tk1,tk3,y
c
c-----------------------------------------------

      dimension y(nequat), drv(nequat)

      stept2 = 0.5*stept
      tk1 = time + stept2
      do i = 1, nequat
         yk1(i) = y(i) + stept2*drv(i)
      enddo
      call timdep(stept,tk1)
      call givedrv(stept,tk1,yk1,drvk2)
      do i = 1, nequat
         yk2(i) = y(i) + stept2*drvk2(i)
      enddo
      call timdep(stept,tk1)
      call givedrv(stept,tk1,yk2,drvk3)
      tk3 = time + stept
      do i = 1, nequat
         yk3(i) = y(i) + stept*drvk3(i)
      enddo
      call timdep(stept,tk3)
      call givedrv(stept,tk3,yk3,drvk4)
      do i = 1, nequat
         y(i) = y(i) + stept*(drv(i)+2.*(drvk2(i)+drvk3(i))+drvk4(i))/6.
      enddo
      return
      end
      
c=======================================================================
c***********************************************************************
      subroutine initbdo
c***********************************************************************
c=======================================================================

c=======================================================================
c a difference between columns (j) and (j+1) of "tbldrv" corresponds
c to a back difference operation of order 1 on the derivatives between
c time = (norder-j)*stept and time = (norder-j-1)*stept;
c in the following k iterations,
c a difference between columns (j) and (j+1) of "deldrv" corresponds
c to a back difference operation of order k on the derivatives between
c time = (norder-j)*stept and time = (norder-j-1)*stept;
c
c for example, we have with norder = 4 :
c
c  --------+------------+------------+------------+----------------->
c          0.         stept       2.*stept     3.*stept             t
c
c       tbldrv(i,4)  tbldrv(i,3)  tbldrv(i,2)  tbldrv(i,1)=olddrv(i)
c
c  k=1 :   -         deldrv(i,3)  deldrv(i,2)  deldrv(i,1)=oldbdo(i,1)
c  k=2 :   -            -         deldrv(i,2)  deldrv(i,1)=oldbdo(i,2)
c  k=3 :   -            -            -         deldrv(i,1)=oldbdo(i,3)
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/bkdf.common'
      include './com_18/coldrv.common'
      include './com_18/memory.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,j,k
c
c-----------------------------------------------

      do j = 1, norder-1
         do i = 1, nequat
            deldrv(i,j) = tbldrv(i,j) - tbldrv(i,j+1)
         enddo
      enddo
      do i = 1, nequat
         oldbdo(i,1) = deldrv(i,1)
         olddrv(i)   = tbldrv(i,1)
      enddo

      do k = 2, norder-1
         do j = 1, norder-k
            do i = 1, nequat
               deldrv(i,j) = deldrv(i,j) - deldrv(i,j+1)
            enddo
         enddo
         do i = 1, nequat
            oldbdo(i,k) = deldrv(i,1)
         enddo
      enddo
      return
      end

c=======================================================================
c***********************************************************************
      subroutine backdiff(drv)
c***********************************************************************
c=======================================================================

c=======================================================================
c the back differences of the derivatives are computed for present time.
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/memory.common'
      include './com_18/newbdo.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,j
      real*8  drv
c
c-----------------------------------------------      
      dimension drv(nequat)

      do i = 1, nequat
         bdo(i,1) = drv(i) - olddrv(i)
      enddo

      do j = 2, norder
         do i = 1, nequat
            bdo(i,j) = bdo(i,j-1) - oldbdo(i,j-1)
         enddo
      enddo
      return
      end

c=======================================================================
c***********************************************************************
      subroutine bashfor(stept,drv,y,ynew)
c***********************************************************************
c=======================================================================

c=======================================================================
c the explicit adams-bashforth prediction scheme estimates
c the value "ynew" of the vector "y" for next time value.
c=======================================================================

c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/newbdo.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,j
      real*8  cstp,drv,y,ynew
c
c-----------------------------------------------      
      dimension cstp(6), drv(nequat), y(nequat), ynew(nequat)
      real*4 stept

      data cstp /0.5, 0.41667, 0.375, 0.34861, 0.32986, 0.31559/

      do i = 1, nequat
         ynew(i) = y(i) + stept*drv(i)
      enddo
      do j = 1, norder
         do i = 1, nequat
            ynew(i) = ynew(i) + stept*cstp(j)*bdo(i,j)
         enddo
      enddo
      return
      end

c=======================================================================
c***********************************************************************
      subroutine remember(drv)
c***********************************************************************
c=======================================================================

c=======================================================================
c the derivatives and their back differences at previous time are saved.
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/memory.common'
      include './com_18/newbdo.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,j
      real*8  drv
c
c-----------------------------------------------      
      dimension drv(nequat)

      do i = 1, nequat
         olddrv(i)   = drv(i)
      enddo

      do j = 1, norder-1
         do i = 1, nequat
            oldbdo(i,j) = bdo(i,j)
         enddo
      enddo
      return
      end

c=======================================================================
c***********************************************************************
      subroutine moulton(stept,drv,y,ynew)
c***********************************************************************
c=======================================================================

c=======================================================================
c time has been incremented;
c the value "ynew" of the vector "y" is estimated at present time step
c with the implicit adams-moulton correction scheme.
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      include './com_18/parameter.common'
      include './com_18/newbdo.common'
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer i,j
      real*8  cstc,drv,y,ynew
c
c-----------------------------------------------      
      dimension cstc(6), drv(nequat), y(nequat), ynew(nequat)
      real*4 stept

      data cstc /-.5, -.083333, -.041667, -.026389, -.01875, -.014269/

      do i = 1, nequat
         ynew(i) = y(i) + stept*drv(i)
      enddo
      do j = 1, norder
         do i = 1, nequat
            ynew(i) = ynew(i) + stept*cstc(j)*bdo(i,j)
         enddo
      enddo
      return
      end
